#!/bin/sh

# Copyright (c) 2016 Samsung Electronics Co., Ltd All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Author: Aleksander Mistewicz <a.mistewicz@samsung.com>

export TSP_DIR="$(CDPATH='' cd -- "$(dirname -- "$0")" && pwd -P)/.."

. "${TSP_DIR}/common.sh"

BUILD_NR="$1"
TARGET="$2"

test -n "$BUILD_NR" || die "Missing argument: build nr"
test -n "$TARGET" || die "Missing argument: target"
test -d "${WS_PUBLISH}" || mkdir -p "${WS_PUBLISH}"

cd "${WS_PUBLISH}"

IMG_WORKDIR="${WS_TEST}/${BUILD_NR}"
DWN_WORKDIR="${WS_DOWNLOAD}/${BUILD_NR}"
WORKSPACE="/home/jenkins/pub"

get_value_of_key_from_dict() {
    KEY="$1"
    DICT="$2"

    test -n "$KEY" || die "Missing argument: key"
    test -n "$DICT" || die "Missing argument: dict"

    VALUE="$(awk -v KEY="${KEY}" '$1 == KEY {print $2}' "$DICT")"

    echo "$VALUE"
}

get_all_pairs() {
    FILE="$1"
    PREFIX="$2"

    test -n "$FILE" || die "Missing argument: file"
    test -n "$PREFIX" || die "Missing argument: prefix"
    test -f "$FILE" || die "No such file: $FILE"

    for key in "RETRIGGER_CNT" "FAILED" "CONNECT_CNT" "OPT_CRASH_CNT" \
        "JOURNAL_TOTAL_CNT" "JOURNAL_DIFF_CNT" "SERVICES_FAIL_CNT"
    do
        # FILE is read every time because sh does not preserve file endings (zsh does)
        eval "${PREFIX}${key}"=$(get_value_of_key_from_dict "$key" "$FILE")
    done
}

compare_with_snapshot() {
    test -f "$SYSCTL_FILE" || return
    test -f "$SNAPSHOT_SYSCTL_FILE" || return

    get_all_pairs "$SYSCTL_FILE" "PRE_"
    get_all_pairs "$SNAPSHOT_SYSCTL_FILE" "SNAP_"

    if [ "$PRE_FAILED" -gt 0 ]
    then
        echo "Fail"
        return
    fi

    if [ "$PRE_RETRIGGER_CNT" -gt 0 ]
    then
        echo "Unclear"
        return
    fi

    for key in "SERVICES_FAIL_CNT" "CONNECT_CNT" "OPT_CRASH_CNT" "SERVICES_FAIL_CNT"
        #"JOURNAL_TOTAL_CNT" "JOURNAL_DIFF_CNT"
    do
        PRE_KEY="PRE_$key"
        PRE_VAL="$(($PRE_KEY))"
        SNAP_KEY="SNAP_$key"
        SNAP_VAL="$(($SNAP_KEY))"
        echo "[$key] snap_value: $SNAP_VAL, pre_value: $PRE_VAL" >&2
        if [ "$PRE_VAL" -gt "$SNAP_VAL" ];
        then
            echo "Unclear"
            return
        fi
    done
    echo "OK"
}

PROJECT_FILE="${DWN_WORKDIR}/project-${TARGET}.conf"
DIFF_FILE="${DWN_WORKDIR}/diff-${TARGET}.report"
SYSCTL_FILE="${IMG_WORKDIR}/sysctl.result"
SNAPSHOT_SYSCTL_FILE="${WS_TEST}/snapshot_sysctl_${TARGET}"
TAG_FILE="${IMG_WORKDIR}/tag"

if grep -q "Snapshot" "${DIFF_FILE}"
then
    echo "Processing snapshot: skip comparison, link new results"
    ln -Tsf "$SYSCTL_FILE" "$SNAPSHOT_SYSCTL_FILE"
    TAG="Snapshot"
else
    echo "Processing prerelease"
    if grep -q "identical" "${DIFF_FILE}"
    then
        echo "Prerelease is identical with snapshot"
        TAG="Identical"
    else
        echo "Compare sysctl with latest tested snapshot"
        TAG=$(compare_with_snapshot)
    fi
fi

test -n "$TAG" || TAG="Unclear"
echo "$TAG" > "$TAG_FILE"

init_db() {
    # Load schema
    sqlite3 -batch "${DBPATH}" < "${TSP_DIR}/results-schema.sql"
    # Load initial values to tables 'device' and 'result'
    sqlite3 -batch "${DBPATH}" < "${TSP_DIR}/results-values-device.sql"
    sqlite3 -batch "${DBPATH}" < "${TSP_DIR}/results-values-result.sql"
}

test -f "${DBPATH}" || init_db

SR=$(head -n 1 "${PROJECT_FILE}")

echo "SR: $SR"
echo "TAG: $TAG"
echo "TARGET: $TARGET"

# Publish to database
sqlite3 "${DBPATH}" "insert into test values (NULL,'$SR', CURRENT_TIMESTAMP, $BUILD_NR,\
    (select r.rid from result r where r.rname='$TAG'),\
    (select d.did from device d where d.dname='$TARGET'));"

# Publish to www
{
    cat "${TSP_DIR}/html_head"
    "${TSP_DIR}/scripts/publish.py" "${DBPATH}"
    cat "${TSP_DIR}/html_tail"
} > "${WWW_PUBLISH}"

# Clean up workspace
ssh tl-pwb "rm -rf \"${WORKSPACE}\"; mkdir -p \"${WORKSPACE}\"" || echo "rm failed"

# Copy required files and ignore the problems
scp "${PROJECT_FILE}" tl-pwb:${WORKSPACE}/project.conf || echo "No project"
scp "${DIFF_FILE}" tl-pwb:${WORKSPACE}/diff.report || echo "No diff"
scp "${SYSCTL_FILE}" tl-pwb:${WORKSPACE}/sysctl.result || echo "No sysctl"
scp "${TAG_FILE}" tl-pwb:${WORKSPACE}/tag || echo "No tag"

# Run the local script on remote machine
ssh tl-pwb "sh -s --" < "${TSP_DIR}/scripts/publish.sh"
